#
# This is the SPEC file for creating binary and source RPMs for the VMs.
#
#
# The Qubes OS Project, http://www.qubes-os.org
#
# Copyright (C) 2010  Joanna Rutkowska <joanna@invisiblethingslab.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
#


# default value in case of no qubes-builder's one

Name:		qubes-gui-agent
Version:	4.2.7
Release:	1%{dist}
Summary:	The Qubes GUI Agent for AppVMs

Group:		Qubes
Vendor:		Invisible Things Lab
License:	GPL
URL:		https://www.qubes-os.org


%define pa_ver_full %((pkg-config --modversion libpulse 2>/dev/null || echo 0.0) | cut -d "-" -f 1)
%define pa_module_dir %(pkg-config --variable=modlibexecdir libpulse)

BuildRequires:	gcc
BuildRequires:	libX11-devel
BuildRequires:	libXcomposite-devel
BuildRequires:	libXdamage-devel
BuildRequires:	libXfixes-devel
BuildRequires:	libXt-devel
BuildRequires:	libtool-ltdl-devel
BuildRequires:	pulseaudio-libs-devel >= 0.9.21, pulseaudio-libs-devel <= 16.1
BuildRequires:	xorg-x11-server-devel
BuildRequires:	pam-devel
BuildRequires:	qubes-libvchan-devel
BuildRequires:	qubes-gui-common-devel >= 4.1.0
BuildRequires:	qubes-db-devel
BuildRequires:	xen-devel
BuildRequires:	mesa-libgbm-devel
BuildRequires:	systemd-rpm-macros
Requires:	Xorg %(xserver-sdk-abi-requires ansic)
Requires:	Xorg %(xserver-sdk-abi-requires videodrv)
Requires:	libXfixes
Requires:	qubes-core-vm >= 3.0.14
Requires:	xorg-x11-xinit
# qubesdb-read --wait option
Requires:	qubes-db >= 4.1.4
Requires:	python%{python3_pkgversion}-xcffib
Requires:   xorg-x11-server-Xorg
Requires:   xorg-x11-server-Xephyr
Requires:   setxkbmap
Requires:   xsetroot
Requires:   xrdb
Requires:   xmodmap
Requires:   xev
Requires:   xdpyinfo
Requires:   xprop
Requires:   xrandr

Provides:   qubes-gui-vm = %{version}-%{release}
Obsoletes:  qubes-gui-vm < 4.0.0

Source0: %{name}-%{version}.tar.gz

Requires: (%{name}-selinux if selinux-policy)

%package -n pulseaudio-qubes
Summary: Audio support for Qubes VM
# The vchan sink needs .h files from pulseaudio sources
# that are not exported by any *-devel packages; thus they are internal and
# possible to change across version. They are copied to gui git.
# It is possible that our code will work fine with any later pulseaudio
# version; but this needs to be verified for each pulseaudio version.
Requires:	pulseaudio = %{pa_ver_full}
# Needed so that systemd can manage PulseAudio
Conflicts:	qubes-core-agent < 4.2.5
Conflicts:  qubes-gui-vm < 4.0.0

%description -n pulseaudio-qubes
Pulseaudio module to enable sound support in Qubes VM

%package -n pipewire-qubes
Summary:       Experimental PipeWire audio support for Qubes VMs
Requires:      pipewire
BuildRequires: pipewire-devel
BuildRequires: qubes-db-devel
BuildRequires: qubes-libvchan-devel

%description -n pipewire-qubes
Experimental PipeWire module to enable sound support in Qubes VMs.
Unlike the PulseAudio support in the pulseaudio-qubes package, this
works with the stock PipeWire service and does not depend on internal
PipeWire headers.  It also promises lower latency than PulseAudio,
at least once the audio daemon supports low latency.

%package xfce
Summary: XFCE desktop support for Qubes VM

%description xfce
XFCE desktop support for Qubes VM

%package -n qubes-gui-vnc
Summary: Remote GuiVM support using VNC
Requires: x11vnc

%description -n qubes-gui-vnc
Remote GuiVM support using VNC

%description
The Qubes GUI agent that needs to be installed in VM in order to provide the
Qubes fancy GUI.

%prep
%setup -q

rm -f pulse/pulsecore
ln -s pulsecore-%{pa_ver_full} pulse/pulsecore

%build
%{?set_build_flags}
#make clean
make appvm

%install
rm -rf -- "$RPM_BUILD_ROOT"
make install "DESTDIR=$RPM_BUILD_ROOT" \
                     LIBDIR=%{_libdir} \
                     DATADIR=%{_datadir} \
                     SYSLIBDIR=/usr/lib \
                     PA_VER_FULL=%{pa_ver_full} \
                     ENABLE_SELINUX=%{with selinux} \
                     UNITDIR=%{_unitdir} \
                     USERUNITDIR=%{_userunitdir} \
                     USERPRESETDIR=%{_userpresetdir} \
                     UDEVRULESDIR=%{_udevrulesdir}

%post
if [ -x /bin/systemctl ] && readlink /sbin/init | grep -q systemd; then
    /bin/systemctl enable qubes-gui-agent.service 2> /dev/null
fi

%post -n pulseaudio-qubes

sed -i '/^autospawn/d' /etc/pulse/client.conf
echo autospawn=no >> /etc/pulse/client.conf

# simplified version of preset_units() from core-agent-linux
preset_units() {
    local represet=
    cat "$1" | while read action unit_name
    do
        if [ "$action" = "#" -a "$unit_name" = "Units below this line will be re-preset on package upgrade" ]
        then
            represet=1
            continue
        fi
        echo "$action $unit_name" | grep -q '^[[:space:]]*[^#;]' || continue
        [ -n "$action" -a -n "$unit_name" ] || continue
        if [ "$2" = "initial" -o "$represet" = "1" ]
        then
            systemctl --no-reload --global preset "$unit_name" >/dev/null 2>&1 || :
        fi
    done
}

if [ $1 -eq 1 ]
then
    preset_units /usr/lib/systemd/user-preset/75-pulseaudio-qubes.preset initial
else
    preset_units /usr/lib/systemd/user-preset/75-pulseaudio-qubes.preset upgrade
fi

%post -n pipewire-qubes
%systemd_user_post pipewire.service pipewire.socket wireplumber.service

%preun -n pipewire-qubes
%systemd_user_preun pipewire.service pipewire.socket wireplumber.service

%preun
if [ "$1" = 0 ] ; then
	chkconfig qubes-gui-agent off
    [ -x /bin/systemctl ] && /bin/systemctl disable qubes-gui-agent.service
    /usr/bin/glib-compile-schemas %{_datadir}/glib-2.0/schemas &> /dev/null || :
fi

%posttrans
    /usr/bin/glib-compile-schemas %{_datadir}/glib-2.0/schemas &> /dev/null || :

if [ -d '/etc/init.d' ] && ! [ "$(ls -A /etc/init.d/ 2> /dev/null)" ]; then
    rmdir /etc/init.d
fi

%triggerin -n pulseaudio-qubes -- pulseaudio-libs

sed -i '/^autospawn/d' /etc/pulse/client.conf
echo autospawn=no >> /etc/pulse/client.conf

%clean
rm -rf $RPM_BUILD_ROOT
rm -f %{name}-%{version}

%files
%defattr(-,root,root,-)
%_bindir/qubes-gui
%_bindir/qubes-gui-runuser
%_bindir/qubes-session
%_bindir/qubes-run-xorg
%_bindir/qubes-run-xephyr
%_bindir/qubes-start-xephyr
%_bindir/qubes-change-keyboard-layout
%_bindir/qubes-set-monitor-layout
%_libdir/xorg/modules/drivers/qubes_drv.so
%_libdir/xorg/modules/drivers/dummyqbs_drv.so
%_libdir/libxf86-qubes-common.so
%attr(0644,root,root) /etc/X11/xorg-qubes.conf.template
/etc/profile.d/qubes-gui.sh
/etc/profile.d/qubes-gui.csh
%config /etc/pam.d/qubes-gui-agent
%config /etc/security/limits.d/90-qubes-gui.conf
%config /etc/xdg/Trolltech.conf
/etc/X11/xinit/xinitrc.d/20qt-x11-no-mitshm.sh
/etc/X11/xinit/xinitrc.d/20qt-gnome-desktop-session-id.sh
/etc/X11/xinit/xinitrc.d/50guivm-windows-prefix.sh
/etc/qubes-rpc/qubes.SetMonitorLayout
/etc/qubes-rpc/qubes.GuiVMSession
/etc/qubes/post-install.d/20-qubes-guivm-gui-agent.sh
%config /etc/sysconfig/desktop
%_unitdir/qubes-gui-agent.service
%_udevrulesdir/70-master-of-seat.rules
%_sysctldir/30-qubes-gui-agent.conf
/usr/lib/qubes/gtk4-workarounds.sh
/usr/lib/qubes/qubes-gui-agent-pre.sh
/usr/lib/qubes/qubes-keymap.sh
/usr/lib/qubes/qubes-xorg-wrapper
%{_datadir}/glib-2.0/schemas/20_qubes-gui-vm.gschema.override
/usr/lib/qubes/icon-sender
/etc/xdg/autostart/qubes-gtk4-workarounds.desktop
/etc/xdg/autostart/qubes-icon-sender.desktop
/etc/xdg/autostart/qubes-qrexec-fork-server.desktop
/etc/xdg/autostart/qubes-keymap.desktop
%dir /var/log/qubes

%files -n pulseaudio-qubes
%_tmpfilesdir/qubes-pulseaudio.conf
/etc/pulse/qubes-default.pa
%_bindir/start-pulseaudio-with-vchan
%{pa_module_dir}/module-vchan-sink.so
/etc/xdg/autostart/qubes-pulseaudio.desktop
/usr/lib/systemd/user-preset/75-pulseaudio-qubes.preset
/usr/lib/systemd/user/pulseaudio.service.d/30_qubes.conf

%files -n pipewire-qubes
%_datadir/pipewire/pipewire.conf.d/30_qubes.conf
%_userpresetdir/74-qubes-vm.preset
%_libdir/pipewire-0.3/libpipewire-module-qubes.so
%_defaultlicensedir/pipewire-qubes/COPYING
/etc/qubes/post-install.d/20-qubes-pipewire.sh

# Works with ordinary PipeWire service :)

%files xfce
/etc/X11/xinit/xinitrc.d/60xfce-desktop.sh

%files -n qubes-gui-vnc
#/lib/systemd/system/qubes-gui-vncserver.service
%attr(0644,root,root) /etc/X11/xorg-qubes-x11vnc.conf.template
%attr(0644,root,root) /etc/systemd/system/lightdm.service.d/qubes-guivm-vnc.conf
%_bindir/qubes-run-x11vnc

%changelog
* Tue May 16 2023 Qubes OS Team <qubes-devel@googlegroups.com>
- For complete changelog see: https://github.com/QubesOS/qubes-gui-agent-linux

* Tue May 16 2023 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - 9d262c4
- version 4.2.7

* Tue May 16 2023 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - 1aad0f6
- deb: Fix clearing source commands

* Tue May 16 2023 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - e81afe3
- version 4.2.6

* Tue May 16 2023 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - 644a0b0
- Cleanup unused file

* Mon May 15 2023 Frédéric Pierret (fepitre) <frederic.pierret@qubes-os.org> - 471c7ac
- Update CI exclude for install

* Mon May 15 2023 Frédéric Pierret (fepitre) <frederic.pierret@qubes-os.org> - c4460b7
- pipewire: drop service vm to be set

* Mon May 15 2023 Frédéric Pierret (fepitre) <frederic.pierret@qubes-os.org> - 3d9b94a
- Step back for jammy on pipewire

* Sun May 07 2023 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - ad04750
- version 4.2.5

* Sun May 07 2023 Frédéric Pierret (fepitre) <frederic.pierret@qubes-os.org> - c5e8d77
- debian: move piprewire-pulse in Recommends

* Sun May 07 2023 Frédéric Pierret (fepitre) <frederic.pierret@qubes-os.org> - aad0b54
- debian: update maintainer

* Sun May 07 2023 Frédéric Pierret (fepitre) <frederic.pierret@qubes-os.org> - af6700d
- debian: enable pipewire starting from bookworm

* Thu May 04 2023 Frédéric Pierret (fepitre) <frederic.pierret@qubes-os.org> - bdf04d9
- pipewire-qubes: add pipewire-pulse dependency

* Tue May 02 2023 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - 8a831cb
- Fix setting pipewire-qubes as recommended under Ubuntu jammy

* Tue May 02 2023 Frédéric Pierret (fepitre) <frederic.pierret@qubes-os.org> - b83e4b5
- jammy: use pipewire

* Tue May 02 2023 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - 948422a
- pulse: use pulseaudio version from pkg-config

* Tue May 02 2023 Frédéric Pierret (fepitre) <frederic.pierret@qubes-os.org> - e404e5e
- .qubesbuilder: fix sed for ubuntu

* Fri Apr 28 2023 Frédéric Pierret (fepitre) <frederic.pierret@qubes-os.org> - 5e8e1d1
- Add support for Ubuntu in builderv2

* Wed Apr 26 2023 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - c1e8d1d
- version 4.2.4

* Wed Apr 26 2023 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - cf245e7
- Add pulse 15.99.1 headers for jammy build

* Fri Apr 14 2023 Demi Marie Obenour <demi@invisiblethingslab.com> - ab5cd1f
- PipeWire agent is not less code

* Fri Apr 14 2023 Demi Marie Obenour <demi@invisiblethingslab.com> - 5dceb10
- PipeWire agent: Add dependency on PipeWire

* Fri Apr 14 2023 Demi Marie Obenour <demi@invisiblethingslab.com> - 9a5d494
- Fix 'make install'

* Fri Apr 14 2023 Demi Marie Obenour <demi@invisiblethingslab.com> - ffed70d
- Advertise the PipeWire service to dom0

* Fri Apr 14 2023 Demi Marie Obenour <demi@invisiblethingslab.com> - 1a02811
- Turn off PulseAudio if PipeWire is enabled

* Fri Apr 14 2023 Demi Marie Obenour <demi@invisiblethingslab.com> - 147038f
- Check that atomics are lock-free

* Fri Apr 14 2023 Demi Marie Obenour <demi@invisiblethingslab.com> - cfe9236
- Avoid using bitfields

* Fri Apr 14 2023 Demi Marie Obenour <demi@invisiblethingslab.com> - 5998085
- Allow domain IDs up to INT_MAX/2

* Fri Apr 14 2023 Demi Marie Obenour <demi@invisiblethingslab.com> - f8acb0d
- Use (C) instead of © in COPYING

* Fri Apr 14 2023 Demi Marie Obenour <demi@invisiblethingslab.com> - 88c0bb9
- Debian packaging fixups

* Fri Apr 14 2023 Demi Marie Obenour <demi@invisiblethingslab.com> - 75cda13
- Assert that SPA builder succeeded

* Fri Apr 14 2023 Demi Marie Obenour <demi@invisiblethingslab.com> - 62096af
- Use a smaller SPA builder buffer

* Fri Apr 14 2023 Demi Marie Obenour <demi@invisiblethingslab.com> - aece0a6
- Fix NULL pointer dereference if vchan connection fails

* Fri Apr 14 2023 Demi Marie Obenour <demi@invisiblethingslab.com> - 7e1c6ec
- Shut down dead streams

* Fri Apr 14 2023 Demi Marie Obenour <demi@invisiblethingslab.com> - 53f21df
- Remove shutting_down global variable

* Fri Apr 14 2023 Demi Marie Obenour <demi@invisiblethingslab.com> - 90c2d0d
- Use common code for stream shutdown

* Fri Apr 14 2023 Demi Marie Obenour <demi@invisiblethingslab.com> - e7e8659
- Provide default buffer sizes

* Fri Apr 14 2023 Demi Marie Obenour <demi@invisiblethingslab.com> - 0223d63
- Process control commands if an overrun occurs

* Fri Apr 14 2023 Demi Marie Obenour <demi@invisiblethingslab.com> - ffbb586
- Sync stream states after vchan open

* Fri Apr 14 2023 Demi Marie Obenour <demi@invisiblethingslab.com> - 28eb0d6
- Do not disable streams if control vchan is closed

* Fri Apr 14 2023 Demi Marie Obenour <demi@invisiblethingslab.com> - f8b8bdc
- Only use pw_log_error() for true errors

* Fri Apr 14 2023 Demi Marie Obenour <demi@invisiblethingslab.com> - 36b24d4
- Do not rely on asserts for error handling

* Fri Apr 14 2023 Demi Marie Obenour <demi@invisiblethingslab.com> - 92e51ff
- Report stream states on vchan connect

* Fri Apr 14 2023 Demi Marie Obenour <demi@invisiblethingslab.com> - eeaec6c
- Shorten the name of the module

* Fri Apr 14 2023 Demi Marie Obenour <demi@invisiblethingslab.com> - 53252f6
- Update usage information

* Fri Apr 14 2023 Demi Marie Obenour <demi@invisiblethingslab.com> - d83b6a8
- Ensure that assertions are enabled

* Fri Apr 14 2023 Demi Marie Obenour <demi@invisiblethingslab.com> - 3b84672
- Fix build with old PipeWire

* Fri Apr 14 2023 Demi Marie Obenour <demi@invisiblethingslab.com> - ebbe882
- Try to handle configuration overrides

* Fri Apr 14 2023 Demi Marie Obenour <demi@invisiblethingslab.com> - fa88d0f
- Distribute license text with package

* Fri Apr 14 2023 Demi Marie Obenour <demi@invisiblethingslab.com> - 914ab04
- Packaging fixups

* Fri Apr 14 2023 Demi Marie Obenour <demi@invisiblethingslab.com> - 0c322d9
- Use less strict number parsing

